Some more details of styles.
[pspp] / dump.c
diff --git a/dump.c b/dump.c
index 04f38e0a751bf41747130472385c1febe6c8c96f..4d7d7f468e694bfa11a59900f45c46920dd84146 100644 (file)
--- a/dump.c
+++ b/dump.c
@@ -798,13 +798,13 @@ hex_dump(int ofs, int n)
   for (int i = 0; i < n; i++)
     {
       int c = data[ofs + i];
-#if 0
+#if 1
       if (i && !(i % 16))
         fprintf(stderr, "-");
       else
         fprintf(stderr, " ");
 #endif
-      fprintf(stderr, "%02x ", c);
+      fprintf(stderr, "%02x", c);
       //fprintf(stderr, "%c", c >= 32 && c < 127 ? c : '.');
     }
   fprintf(stderr, "\n");
@@ -949,11 +949,10 @@ dump_fonts(void)
     {
       int outer_end = get_end();
 
-      /* Skip first inner envelope. */
-      int inner_len = get_u32();
-      hex_dump(pos, inner_len);
-      pos += inner_len;
+      /* First inner envelope: byte*33 int[n] int*[n]. */
+      pos = get_end();
 
+      /* Second inner envelope. */
       assert(get_end() == outer_end);
 
       match_byte_assert(1);
@@ -1001,25 +1000,26 @@ dump_fonts(void)
 
       if (outer_end - pos > 6)
         {
-          uint8_t *endp = memmem(&data[pos], outer_end - pos, "\5\0\0\0\4", 5);
-          assert(endp);
-
-          int end = endp - data;
-          if (pos != end)
+          /* There might be a pair of strings representing a dataset and
+             datafile name, or there might be a set of custom currency strings.
+             The custom currency strings start with a pair of integers, so we
+             can distinguish these from a string by checking for a null byte; a
+             small 32-bit integer will always contain a null and a text string
+             never will. */
+          int save_pos = pos;
+          int len = get_u32();
+          bool has_dataset = !memchr(&data[pos], '\0', len);
+          pos = save_pos;
+
+          if (has_dataset)
             {
               printf("<dataset>%s</dataset>\n", get_string());
               printf("<datafile>%s</datafile>\n", get_string());
 
-              if (pos != end)
-                {
-                  match_u32_assert(0);
-                  get_u32();
-                  match_u32_assert(0);
-                }
+              match_u32_assert(0);
+              get_u32();
+              match_u32_assert(0);
             }
-          //fprintf(stderr, "%d\n", end - pos);
-          //hex_dump(pos, end - pos);
-          assert(pos == end);
         }
 
       if (match_u32(5))
@@ -1039,7 +1039,7 @@ dump_fonts(void)
       pos = outer_end;
     }
   else
-    pos = get_end();
+    match_u32_assert(0);
 }
 
 int